# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::TeX::TeX;

use strict;
use Encode;
use Exporter;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		tex_encode
		make_pdf_file
		make_ps_file
		set_tex_config
		);
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Common::Constantes;
use FDN::Common::Utils;
use FDN::Web::TemplateParse;

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

my $conf = 'tex.cfg';
my $basedir = '/tmp';
my $tpldir = '/tmp';

sub set_tex_config($);

sub set_tex_config($) {
	my ( $file ) = @_;
	my $cfg = Load_Config($file);
	if ( ! defined $cfg ) {
		error "Failed to load configuration file $file.";
		return FALSE;
	}
	if ( ! defined $cfg->{'common'} ) {
		error "No [common] section in $file.";
		return FALSE;
	}
	if ( ! defined $cfg->{'common'}->{'basedir'} ) {
		error "No 'basedir' entry in [common] section of $file.";
		return FALSE;
	}
	if ( ! defined $cfg->{'common'}->{'tpldir'} ) {
		error "No 'tpldir' entry in [common] section of $file.";
		return FALSE;
	}
	$conf = $file;
	$basedir = $cfg->{'common'}->{'basedir'};
	$tpldir  = $cfg->{'common'}->{'tpldir'};
	__debug "Successfuly loaded config file: $file.";
	return TRUE;
}

BEGIN {
	if ( -f $FDN::Common::Config::CONFIG_ROOT.'/tex.cfg' ) {
		info "Preloading default config.";
		set_tex_config('tex.cfg');
	} else {
		info "No default config file.";
	}
}

sub get_doc_dir($) {
	my ( $doc ) = @_;
	$doc = '' if ! defined $doc;
	my $cfg = Config_Key($conf,$doc,'dir');
	if ( ! defined $cfg ) {
		$cfg = '.';
	}
	__debug "Local directory for document '$doc' is '$cfg'.";
	return $cfg;
}

sub get_doc_template($) {
	my ( $doc ) = @_;
	$doc = '' if ! defined $doc;
	my $cfg = Config_Key($conf,$doc,'tpl');
	if ( ! defined $cfg ) {
		$cfg = $doc.'.tpl';
	}
	__debug "Template file for document '$doc' is '$cfg'.";
	return $cfg;
}

sub get_doc_pdf($) {
	my ( $doc ) = @_;
	my $cfg = Config_Key($conf,$doc,'pdf');
	if ( ! defined $cfg ) {
		$cfg = $doc.'.pdf';
	}
	__debug "Resulting PDF file for document '$doc' is '$cfg'.";
	return $cfg;
}

sub get_doc_tex($) {
	my ( $doc ) = @_;
	my $cfg = Config_Key($conf,$doc,'doc');
	if ( ! defined $cfg ) {
		$cfg = get_doc_pdf($doc);
		$cfg =~ s/pdf$/tex/;
	}
	__debug "Source TeX file to be created for document '$doc' is '$cfg'.";
	return $cfg;
}

sub get_doc_fichiers($) {
	my ( $doc ) = @_;
	my $cfg = Config_Key($conf,$doc,'@fichiers');
	if ( ! defined $cfg ) {
		$cfg = [];
	}
	if ( ref($cfg) ne 'ARRAY' ) {
		error '@fichiers for a TeX document should be an array!';
		$cfg = [];
	}
	__debug "The files to be copied for compiling document '$doc' are '".@$cfg."'.";
	return $cfg;
}

sub get_doc_nbrun($) {
	my ( $doc ) = @_;
	my $cfg = Config_Key($conf,$doc,'nbrun');
	if ( ! defined $cfg ) {
		$cfg = 1;
	}
	__debug "The number of runs for document '$doc' is '$cfg'.";
	return $cfg;
}

sub make_the_file($$$$) {
	my ( $doc, $dir, $hash, $proc ) = @_;

	my $dstdir = $basedir.'/'.get_doc_dir($doc).'/'.$dir;
	my $template = $tpldir.'/'.get_doc_template($doc);
	my $file   = $dstdir.'/'.get_doc_tex($doc);
	my $pdf    = $dstdir.'/'.get_doc_pdf($doc);
	my $files  = get_doc_fichiers($doc);
	my $nbrun  = get_doc_nbrun($doc);

	my $output = $file;
	my $ext;
	if ( $proc eq 'latex' ) {
		$ext = 'dvi';
	} else {
		$ext = 'pdf';
	}
	$output =~ s/tex$/$ext/;
	
#	print "Destination: $dstdir\n";
#	print "Fichier:     $file\n";
#	print "Template:    $tpl\n";
#	print "PDF a prod.: $pdf\n";
	
	if ( ! -d "$dstdir" ) {
		mkdir_p($dstdir);
		__debug "Created directory $dstdir (not checked yet)."
	}
	if ( ! -d "$dstdir" ) {
		error "Failed to create $dstdir";
		return undef;
	}
	my $content = remplace("$template",$hash,1);
	if ( ! open OUTPUT, ">$file" ) {
		error "Failed to open TeX file $file.";
		return undef;
	}
	print OUTPUT $content;
	close OUTPUT;
	__debug "TeX document is ready in file $file.";

	my $commande;
	for my $f ( @$files ) {
		$commande .= "cp $tpldir/$f $dstdir; ";
	}
	$commande .= "cd $dstdir; ";
	$commande .= "$proc $file < /dev/null > /dev/null 2>&1;" x $nbrun;
	if ( $proc eq 'latex' ) {
		$commande .= "dvips $output -o $pdf";
	} elsif ( $output ne $pdf ) {
		$commande .= "cp $output $pdf;";
	}
	__debug "Going to run that command to build the PDF file: ".$commande;
	system($commande);
	__debug "TeX document have been compiled.";
	if ( -s $pdf ) {
		__debug "Compilation successfull, result is PDF file $pdf.";
		return $pdf;
	} else {
		__debug "Compilation failed. Check the log files.";
		return undef;
	}
}

sub make_ps_file($$$) {
	my ( $doc, $dir, $hash ) = @_;
	return make_the_file($doc,$dir,$hash,'latex');
}

sub make_pdf_file($$$) {
	my ( $doc, $dir, $hash ) = @_;
	return make_the_file($doc,$dir,$hash,'pdflatex');
}

sub tex_encode($) {
	my ( $texte ) = @_;
	return undef if ! defined $texte;

	my $latin = 0;
	if ( ! Encode::is_utf8($texte) ) {
		Encode::_utf8_on($texte);
	}
	if ( ! utf8::valid($texte) ) {
		Encode::_utf8_off($texte);
		$texte = Encode::decode("iso-8859-1",$texte);
		$latin = 1;
	}
		
	$texte =~ s/\\/\//g;
	$texte =~ s/([_&{}\$#%])/\\$1/g;
	$texte =~ s/\^/\^{}/g;
	if ( $texte =~ /^[\x{0000}-\x{024f}\x{2c60}-\x{2c7f}]*$/ms ) {
		# 0x0000 - 0x00ff : latin 1
		# 0x0100 - 0x024f : latin extended A et latin extended B
		# 0x2c60 - 0x2c7f : latin extended C
		if ( $latin ) {
			$texte = Encode::encode("iso-8859-1",$texte);
			$texte =~ s/[\x{0080}-\x{009f}]//g;
		}
		return $texte;
	}
	if ( $texte =~ /[\x{0400}-\x{0527}]/ms ) {
		# 0x0400 - 0x04ff : Cyrillic
		# 0x0500 - 0x0527 : Cyrillic supplement
		return '\RussianText{'.$texte.'}';
	}
	# Sinon on retire tout ce qui n'est pas latin
	$texte =~ s/[^\x{0000}-\x{024f}\x{2c60}-\x{2c7f}]//g;
	if ( $latin ) {
		$texte = Encode::encode("iso-8859-1",$texte);
	}
	return $texte;
}

1;
